package com.usky.fire.controller.web;


import com.usky.common.core.bean.ApiResult;
import com.usky.fire.service.PatrolInspectionPlanSonService;
import com.usky.fire.service.vo.PatrolInspectionPlanSonVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * APP-巡检
 *
 * @author JCB
 * @since 2022-07-12
 */
@RestController
@RequestMapping("/appPatrolInspection")
public class AppPatrolInspectionController {

    @Autowired
    private PatrolInspectionPlanSonService patrolInspectionPlanSonService;

    /**
     * App-巡检自查-首页
     *
     * @param personnelId 人员ID
     * @return
     */
    @GetMapping("appPlanStatistics")
    public ApiResult<Map<String, Object>> appPlanStatistics(@RequestParam(value = "personnelId") Integer personnelId,
                                                            @RequestParam(value = "currentDate", required = false) String currentDate) {
        return ApiResult.success(patrolInspectionPlanSonService.appPlanStatistics(personnelId, currentDate));
    }


    /**
     * App-巡检自查-巡检计划
     *
     * @param personnelId 人员ID
     * @param currentDate 日期 格式：yyyy-MM-dd
     * @param sort        排序 （正序：ASC 倒序：DESC）
     * @return
     */
    @GetMapping("patrolInspectionPlan")
    public ApiResult<Map<String, Object>> patrolInspectionPlan(@RequestParam(value = "personnelId") Integer personnelId,
                                                               @RequestParam(value = "currentDate", required = false) String currentDate,
                                                               @RequestParam(value = "sort", required = false, defaultValue = "DESC") String sort) {
        return ApiResult.success(patrolInspectionPlanSonService.patrolInspectionPlan(personnelId, currentDate, sort));
    }

    /**
     * 手机端-巡检自检-巡检计划-扫码查询计划
     *
     * @param personnelId 人员ID
     * @param siteNubmber 地点号码
     * @return
     */
    @GetMapping("siteDetails")
    public ApiResult<List<PatrolInspectionPlanSonVo>> siteDetails(@RequestParam(value = "personnelId") Integer personnelId,
                                                                  @RequestParam(value = "siteNubmber") String siteNubmber) {
        return ApiResult.success(patrolInspectionPlanSonService.siteDetails(siteNubmber, personnelId));
    }
}